/* ************************************************************************
*   File: screen.h                                      Part of CircleMUD *
*  Usage: header file with ANSI color codes for online color              *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

/* The standard ANSI colors */
#define COL_NONE	""
#define CNUL  ""
#define COL_OFF 	"\x1B[0;37m"

#define COL_NRM  "\x1B[0;0m"	/* %00 */
#define COL_RED 	"\x1B[0;31m"
#define COL_GREEN 	"\x1B[0;32m"
#define COL_YELLOW 	"\x1B[0;33m"
#define COL_BLUE 	"\x1B[0;34m"
#define COL_MAGENTA "\x1B[0;35m"
#define COL_CYAN 	"\x1B[0;36m"
#define COL_WHITE 	"\x1B[0;37m"
#define COL_BLACK	"\x1B[0;30m"
#define COL_FG_OFF	"\x1B[38m" /* works on normal terminals but not
					color_xterms */

/* The bold or extended ANSI colors */
#define COL_E_RED 		"\x1B[1;31m"
#define COL_E_GREEN 	"\x1B[1;32m"
#define COL_E_YELLOW 	"\x1B[1;33m"
#define COL_E_BLUE 		"\x1B[1;34m"
#define COL_E_MAGENTA 	"\x1B[1;35m"
#define COL_E_CYAN 		"\x1B[1;36m"
#define COL_E_WHITE 	"\x1B[1;37m"
#define COL_E_BLACK		"\x1B[1;30m" /* /cl */

/* The background colors */
	
#define COL_BK_RED 			"\x1B[41m"
#define COL_BK_GREEN 		"\x1B[42m"
#define COL_BK_YELLOW 		"\x1B[43m"
#define COL_BK_BLUE 		"\x1B[44m"
#define COL_BK_MAGENTA 		"\x1B[45m"
#define COL_BK_CYAN 		"\x1B[46m"
#define COL_BK_WHITE 		"\x1B[47m"
#define COL_BK_BLACK    	"\x1B[40m" /* /cL */

/*         Underline, Flashing, etc         */
#define STYLE_UNDERLINE   	"\x1B[4m" /* /cu */
#define STYLE_FLASH	  		"\x1B[5m" /* /cf */
#define BNRM   "\x1B[1m"	/* %27 */
#define STYLE_REVERSE	  	"\x1B[7m" /* /cv */
#define SPACE  " "		/* %28 */
#define BACKSPACE "\x8"*/		/* %29 */
/*#define BACKSPACE	"\x101"*/
/*#define BSMILEY	 "\x101"*/	/* %30 */
/*#define WSMILEY	 "\x102"*/	/* %31 */
 
/* Cursor controls */
#define C_UP    		"\x1B[A"
#define C_DOWN  		"\x1B[B"
#define C_RIGHT 		"\x1B[C"
#define C_LEFT  		"\x1B[D"
#define C_HOME  		"\x1B[H"
#define C_CLR   C_HOME	"\x1B[J"
 
/* Circle's Standard color defines */
#define KNRM  "\033[0m"
#define KRED  "\033[31m"
#define KGRN  "\033[32m"
#define KYEL  "\033[33m"
#define KBLU  "\033[34m"
#define KMAG  "\033[35m"
#define KCYN  "\033[36m"
#define KWHT  "\033[37m"
#define KNUL  ""

/* conditional color.  pass it a pointer to a char_data and a color level. */
#define C_OFF	0
#define C_SPR	1
#define C_NRM	2
#define C_CMP	3
#define _clrlevel(ch) (!IS_NPC(ch) ? (PRF_FLAGGED((ch), PRF_COLOR_1) ? 1 : 0) + \
		       (PRF_FLAGGED((ch), PRF_COLOR_2) ? 2 : 0) : 0)
#define clr(ch,lvl) (_clrlevel(ch) >= (lvl))
#define CCNRM(ch,lvl)  (clr((ch),(lvl))?KNRM:KNUL)
#define CCRED(ch,lvl)  (clr((ch),(lvl))?KRED:KNUL)
#define CCGRN(ch,lvl)  (clr((ch),(lvl))?KGRN:KNUL)
#define CCYEL(ch,lvl)  (clr((ch),(lvl))?KYEL:KNUL)
#define CCBLU(ch,lvl)  (clr((ch),(lvl))?KBLU:KNUL)
#define CCMAG(ch,lvl)  (clr((ch),(lvl))?KMAG:KNUL)
#define CCCYN(ch,lvl)  (clr((ch),(lvl))?KCYN:KNUL)
#define CCWHT(ch,lvl)  (clr((ch),(lvl))?KWHT:KNUL)

#define COLOR_LEV(ch) (_clrlevel(ch))

#define QNRM CCNRM(ch,C_SPR)
#define QRED CCRED(ch,C_SPR)
#define QGRN CCGRN(ch,C_SPR)
#define QYEL CCYEL(ch,C_SPR)
#define QBLU CCBLU(ch,C_SPR)
#define QMAG CCMAG(ch,C_SPR)
#define QCYN CCCYN(ch,C_SPR)
#define QWHT CCWHT(ch,C_SPR)

